package samples;

import com.google.gson.*;

import java.util.*;
import java.io.*;

public class sample_calls {

	/*
	 * Program that demos sample API calls
	 *
	 *
	 * Notes:
	 * - requests.java is required for making the API requests simpler
	 * - requestParser provides code to parse the JSON strings
	 * - MultipartUtility creates multipart/form-data responses
	 * - Switch the baseURL to "https://sign.signority.com/api"
	 * 
	 * @author Felix Yang
	 */
	
	static JsonParser jsonParser = new JsonParser();
	
	public static void main(String[] args) throws IOException{
		// TODO Auto-generated method stub
		String username = "signorityapidemo@gmail.com";
		String password = "password";
		String apiKey = "";
		String authKey = "";
		
		/*
		 * __________________________________________________________
		 * 1. Login
		 * 	   Logging in using username and password
		 * __________________________________________________________
		 * 
		 */
		
		// Making the request with the specified URL
		String login_response = requests.get("https://test.signority.com/api/v1/user/login?email="+username+"&password="+password, null);
		
		// Storing the logged in user's API Key
		apiKey = requestParser.getAPIKey(login_response);
		
		// We can see the String that is being output (Possibly convert to Map object later)
		System.out.println(login_response);
		
		// Generates the Authorization Code for the logged in user
		// If it is known, it can be put in manually and this can be commented out
		authKey = "Basic " + Base64.getUrlEncoder().encodeToString((apiKey + ":").getBytes());
		
		/*
		 * __________________________________________________________
		 * 2. Account Details
    	 * 		After being logged in, you can access the logged in
    	 * 		user's information
    	 * __________________________________________________________
    	 * 
		 */
		
		// Authentication is required to access the account information
		String detail_response = requests.get("https://test.signority.com/api/v1/user/", authKey);
		System.out.println(detail_response);
		
		/*
		 * __________________________________________________________
		 * 3. Create a new Document/Template
		 * 		See documentation for information regarding the data
		 * 		types expected in each field
		 * __________________________________________________________
		 * 
		 */
		
		// Creating a JSON object using a String Manually (Not recommended)
		/*
		JsonObject cdoc_JSON = new JsonObject();
		JsonArray document_array = new JsonArray();
		JsonObject documents = new JsonObject();
		documents.addProperty("title", "API Test Document 001");
		documents.addProperty("senderName", "Java User 27");
		documents.addProperty("senderEmail", "f234yang@edu.uwaterloo.ca");
		documents.addProperty("signingStatus", 51);
		documents.addProperty("documentType", 5);
		
		JsonArray inv = new JsonArray();
		JsonObject inv1 = new JsonObject();
		inv1.addProperty("id", -1);
		inv1.addProperty("recipientName", "Java Recipient");
		inv1.addProperty("recipientEmail", "pythonrecip@mail.com");
		inv1.addProperty("actionType", 15);
		inv1.addProperty("status", 60);
		inv1.addProperty("sequence", 1);
		inv.add(inv1);
		documents.add("invitations", inv);
		document_array.add(documents);
		cdoc_JSON.add("documents", document_array);
		*/
		
		// The above JSON Object corresponds to the JSON below
		String cdocJSONString = "{\r\n" + 
				"    \"documents\": [\r\n" + 
				"        {\r\n" + 
				"            \"title\": \"API Test document 001\",\r\n" + 
				"            \"senderName\": \"Java User 27\",\r\n" + 
				"            \"senderEmail\": \"signorityapidemo@gmail.com\",\r\n" + 
				"            \"signingStatus\": 51,\r\n" + 
				"            \"documentType\": 5,\r\n" + 
				"            \"invitations\": [\r\n" + 
				"                {\r\n" + 
				"                    \"id\": -1,\r\n" + 
				"                    \"recipientName\": \"Java Recipient\",\r\n" + 
				"                    \"recipientEmail\": \"pythonrecip@mail.com\",\r\n" + 
				"                    \"actionType\": 15,\r\n" + 
				"                    \"status\": 60,\r\n" + 
				"                    \"sequence\": 1\r\n" + 
				"                }\r\n" + 
				"            ]\r\n" + 
				"        }\r\n" + 
				"    ]\r\n" + 
				"}";
		// Request using a JSON object
        JsonObject cdoc_JSON = jsonParser.parse(cdocJSONString).getAsJsonObject();
		String cdoc_response = requests.post("https://test.signority.com/api/v1/documents/array", cdoc_JSON.toString(), authKey);
		System.out.println(cdoc_response);
		
		// Set the DocumentID to the recently created Document's ID
		String documentID = requestParser.getDocID(cdoc_response);
		System.out.println(documentID);
		
		/*
		 * __________________________________________________________
		 * 4. Uploading files to a document
		 * 		See documentation for information regarding the data
		 * 		types expected in each field
		 * __________________________________________________________
		 * 
		 */
		
		// Change the path
		File blankFileUpload = new File("projectlocation\\Signority_API_JAVA\\src\\samples\\blank.pdf");
		// What the sample JSON data would look like
		String upload_JSON = "{\r\n" + 
				"    \"addTagsOfPDF\": false,\r\n" + 
				"    \"addAnchorTags\": false,\r\n" + 
				"    \"highDisplayResolution\": false,\r\n" + 
				"    \"files\": {blankFileUpload},\r\n" + 
				"}";
		
		// Next step: Create a parser that automatically parses jsons and makes the request 
		MultipartUtility myForm = new MultipartUtility();
		myForm.MultipartUtilityV2("https://test.signority.com/api/v1/documents/"+documentID+"/files.html", authKey);
		myForm.addFormField("addTagsOfPDF", "false");
		myForm.addFormField("addAnchorTags", "false");
		myForm.addFormField("highDisplayResolution", "false");
		myForm.addFilePart("files", blankFileUpload);
		myForm.addFilePart("files", blankFileUpload);
		System.out.println(myForm.finish());
		
		/*
		 * __________________________________________________________
		 * 5. Updating + Sending a Document
		 * 		Note: When the file is being updated, it must be sent 
		 * 		to the invitees after it's done
		 * 		Note: After the document is sent, modifications
		 * 		to it's contents can longer be made
		 * __________________________________________________________
		 * 
		 */
		
		// The example JSON just sends the documents to the invitees
		String updateJSONString = "{\r\n" + 
				"    \"document\": {\r\n" + 
				"        \"signingStatus\": 51\r\n" + 
				"    }\r\n" + 
				"}";
		
		JsonObject update_JSON = jsonParser.parse(updateJSONString).getAsJsonObject();
		// Printing the outcome of the call and 
		System.out.println(requests.put("https://test.signority.com/api/v1/documents/" + documentID + "/", update_JSON.toString(), authKey));
		
		/*
		 * __________________________________________________________
		 * 6. Update Document Tags
		 * 		Not recommended way to add tags to a document but can
		 * 		be done, the better way is to use Anchor Tags.
		 * 		Anchor Tags: <Link to Anchor Tags Info>
		 * __________________________________________________________
		 * 
		 */
		
		// The example JSON to update document tags
		String uDoc_JSON = "{\r\n" + 
				"	\"tags\": [\r\n" + 
				"		{\r\n" + 
				"			\"pageIndex\" : 0,\r\n" + 
				"			\"invitationIndex\": 0,\r\n" + 
				"			\"name\" : \"U01\",\r\n" + 
				"			\"isMandatory\": true,\r\n" + 
				"			\"height\":35,\r\n" + 
				"			\"width\":107,\r\n" + 
				"			\"left\": 200,\r\n" + 
				"			\"top\": 200,\r\n" + 
				"			\"type\": 250\r\n" + 
				"		},\r\n" + 
				"		{\r\n" + 
				"			\"pageIndex\": 0,\r\n" + 
				"			\"invitationIndex\": 0,\r\n" + 
				"			\"name\": \"u02\",\r\n" + 
				"			\"isMandatory\": true,\r\n" + 
				"			\"height\": 35,\r\n" + 
				"			\"width\": 107,\r\n" + 
				"			\"left\": 200,\r\n" + 
				"			\"top\": 200,\r\n" + 
				"			\"type\": 250\r\n" + 
				"		}\r\n" + 
				"	]\r\n" + 
				"}";
		
		// Making the API call
		System.out.println(requests.put("https://test.signority.com/api/v1/documents/" + documentID + "/tags/", uDoc_JSON, authKey));
		
		/*
		 * __________________________________________________________
		 * 7. Bulk Sign using a Template
		 * 		Sends the template to many signatures in the
		 * 		invitation list
		 * __________________________________________________________
		 * 
		 */
		
		// Setting up the Bulk Sign JSON
		String bulk_JSON = "{\r\n" + 
				"    \"documentSettings\": {\r\n" + 
				"        \"postBackUrl\": \"http://test.signority.com/ui\"\r\n" + 
				"    },\r\n" + 
				"    \"invitations\": [{\r\n" + 
				"        \"sequence\": 1,\r\n" + 
				"        \"recipientName\": \"Bulk Signer List\",\r\n" + 
				"        \"recipientEmail\": \"bulksign@bulk.com\",\r\n" + 
				"        \"actionType\": 15,\r\n" + 
				"        \"isBulkSigner\": True,\r\n" + 
				"        \"EnableIndicatorManuallyFilledAddress\": True\r\n" + 
				"    }\r\n" + 
				"    ]\r\n" + 
				"}";
		
		// Making the API Call
		System.out.println(requests.post("https://test.signority.com/api/v1/templates/use/" + documentID + '/', bulk_JSON, authKey));
		
		/*
		 * __________________________________________________________
		 * 8. Retrieve Document Data
		 * 		On success it will return the information on the
		 * 		queried document
		 * __________________________________________________________
		 * 
		 */
		
		System.out.println(requests.get("https://test.signority.com/api/v1/documents/" + documentID, authKey));
		
		/*
		 * __________________________________________________________
		 * 9. Get a List of Documents
		 * 		Returns all of the documents that is owned or
		 * 		accessible to the logged in user
		 * __________________________________________________________
		 * 
		 */
		
		System.out.println(requests.get("https://test.signority.com/api/v2/documents/search2", authKey));
		
		/*
		 * __________________________________________________________
		 * 10. Update Invitation List
		 * 		Change all of the invitees on a document/template
		 * __________________________________________________________
		 * 
		 */
		
		String uInv_JSON = "{\r\n" + 
				"    \"invitations\": [\r\n" + 
				"        {\r\n" + 
				"            \"sequence\": 1,\r\n" + 
				"            \"recipientName\": \"API TEST USER01\",\r\n" + 
				"            \"recipientEmail\": \"user01@mail.com\",\r\n" + 
				"            \"actionType\": 15\r\n" + 
				"        },\r\n" + 
				"        {\r\n" + 
				"            \"sequence\": 2,\r\n" + 
				"            \"recipientName\": \"API TEST USER02\",\r\n" + 
				"            \"recipientEmail\": \"user02@mail.com\",\r\n" + 
				"            \"actionType\": 15\r\n" + 
				"        }\r\n" + 
				"    ]\r\n" + 
				"}";
		
		System.out.println(requests.put("https://test.signority.com/api/v1/documents/" + documentID + "/invitations/", uInv_JSON, authKey));
		
		/*
		 * __________________________________________________________
		 * 11. Get a List of Invitees
		 * 		Returns all of the invitees on a given document that
		 * 		the signed in user has access to
		 * __________________________________________________________
		 * 
		 */
		
		System.out.println(requests.get("https://test.signority.com/api/v1/documents/" + documentID + "/invitations/", authKey));
		
		/*
		 * __________________________________________________________
		 * 12. Get List of Files
		 * 		Returns all of the files in a document
		 * __________________________________________________________
		 * 
		 */
		
		System.out.println(requests.get("https://test.signority.com/api/v1/documents/" + documentID + "/files/", authKey));
		
		
		/*
		 * __________________________________________________________
		 * 13. Delete a File
		 * 		Deleted a file from the document
		 * __________________________________________________________
		 * 
		 */
		
		// Switch fileID to the file ID of the file that should be deleted
		int fileID = -1;
		System.out.println(requests.delete("https://test.signority.com/api/v1/documents/" + documentID + "/files/" + fileID, authKey));
	}
}
